(*
    EXMPL03.PAS - Example program for the TGX unit
    Miscellaneous examples
    Copyright (c) 1993-2003,2004 by Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
program Exmpl03;
uses Crt, Dos, Tgx;

const
  MESSAGE       : string[80] = 'press any key to exit'; (* Max 32 chars *)
  (* Gauge meter definitions *)
  METER_RANGE   = 80;
  METER_LENGTH  = 8;
  METER_DATA    : array[ 1..9 ] of TTgxChar = (
    (Code:210; Attr:$03; Data:(0,0,0,0,0,0,56,127,127,127,56,0,0,0,0,0)),
    (Code:211; Attr:$03; Data:(0,0,0,0,0,1,7,255,255,255,7,1,0,0,0,0)),
    (Code:212; Attr:$03; Data:(0,7,31,127,255,255,255,255,255,255,255,255,255,127,31,7)),
    (Code:213; Attr:$3C; Data:(0,0,1,1,1,192,192,192,192,192,192,192,1,1,1,0)),
    (Code:214; Attr:$3C; Data:(0,0,1,1,1,240,240,240,240,240,240,240,1,1,1,0)),
    (Code:215; Attr:$3C; Data:(0,0,1,1,1,252,252,252,252,252,252,252,1,1,1,0)),
    (Code:216; Attr:$3C; Data:(0,0,1,1,1,255,255,255,255,255,255,255,1,1,1,0)),
    (Code:218; Attr:$3C; Data:(0,0,1,1,1,0,0,0,0,0,0,0,1,1,1,0)),
    (Code:219; Attr:$03; Data:(224,240,248,252,252,252,252,252,252,252,252,252,252,248,240,224))
  );

var
  CX, CY: integer;
  RX, RY: real;
  P1, P2, P3: PTgxWindow;
  MeterDelta,
  MeterCount: integer;
  Hour, Min, Sec, CS: word;

procedure Abort( Msg: string );
begin
  TextMode( CO80 );
  WriteLn( Msg );
  Halt( 1 );
end;

function CircleX( Scale: real; Step, Range: word ): integer;
begin
  CircleX := CX+Round( RX*Scale*Cos(-PI/2+2*PI*Step/Range) );
end;

function CircleY( Scale: real; Step, Range: word ): integer;
begin
  CircleY := CY+Round( RY*Scale*Sin(-PI/2+2*PI*Step/Range) );
end;

(* Draws a line (slow, but easiest way I could find...) *)
procedure Line( P: PTgxWindow; X1, Y1, X2, Y2: integer );
var
  I: integer;
begin
  for I:=1 to CX do
    tgxSetPixel( P, X1+I*(X2-X1) div CX, Y1+I*(Y2-Y1) div CX );
end;

(* Draws a clock *)
procedure DrawClock( P: PTgxWindow; H, M, S: word );
var
  I: word;
begin
  (* Draw clock *)
  tgxClear( P );
  for I:=0 to 11 do             (* Draw quadrant *)
    tgxSetPixel( P, CircleX( 1, I, 12 ), CircleY( 1, I, 12 ) );
  Line( P, CX, CY, CircleX( 0.9, S, 60 ), CircleY( 0.9, S, 60 ) );
  Line( P, CX, CY, CircleX( 0.8, M, 60 ), CircleY( 0.8, M, 60 ) );
  H := (H mod 12)*30+M shr 1;
  Line( P, CX, CY, CircleX( 0.6, H, 360 ), CircleY( 0.6, H, 360 ) );
end;

(* Draws a wavey message *)
procedure DrawMessage( P: PTgxWindow );
const
  Phase: real = 0.0;
var
  C: TTgxChar;
  PC: PTgxCharArray;
  A, I, CharHeight: word;
begin
  tgxClear( P2 );
  CharHeight := P2^.PixelHeight div P2^.Height;
  PC := P2^.Chars;
  for I:=0 to Length(MESSAGE)-1 do begin
    tgxGrabChar( MESSAGE[I+1], @C );
    A := CharHeight shr 1-Round( CharHeight/2*Sin( Phase+2*PI*I/Length(MESSAGE) ) );
    Move( C.Data[0], PC^[I].Data[A], CharHeight-A );
    Move( C.Data[CharHeight-A], PC^[I+Length(MESSAGE)].Data, A );
  end;
  Phase := Phase + ( PI/Length(MESSAGE) );
end;

(* Draws a gauge meter, thes show it *)
procedure DrawAndShowMeter( P: PTgxWindow; Count: word );
var
  S: array[ 0..METER_LENGTH+3 ] of byte;
  I, Q: word;
begin
  S[0] := 0;
  S[1] := 1;
  S[2] := 2;
  for I:=1 to METER_LENGTH do
    S[2+I] := 7;
  S[METER_LENGTH+3] := 8;
  Q := (Count mod METER_RANGE)*METER_LENGTH*4 div METER_RANGE;
  for I:=1 to Q div 4 do
    S[2+I] := 6;
  if( Q mod 4 > 0 ) then
    S[Q div 4+3] := 2+Q mod 4;
  for I:=0 to SizeOf(S)-1 do
    MemW[ SegB800:P^.Y*160+(I+P^.X)*2 ] := P^.Chars^[S[I]].Code+word(P^.Chars^[S[I]].Attr) shl 8;
end;

var
  C: char;

begin
  (* Initialize the library *)
  TextMode( CO80 );
  if( tgxInit <> 0 ) then
    Abort( 'Cannot initialize TGX library.' );

  (* Create clock window *)
  P1 := tgxCreate( 0, 0,                        (* Position *)
                   6, 3,                        (* Size *)
                   Yellow,                      (* Attribute *)
                   pointer(longint($C0))        (* Base character *)
                 );
  (* Create message window *)
  P2 := tgxCreate( 34, 4,
                   Length(MESSAGE), 2,
                   LightCyan,
                   pointer(longint($80))
                 );
  (* Create "dummy" meter window *)
  P3 := tgxCreate( 12, 10,
                   9, 1,
                   0,
                   NIL                          (* Do not init *)
                 );
  if( P1 = NIL )or( P2 = NIL )or( P3 = NIL ) then
    Abort( 'Cannot create window.' );

  (* Clear and show windows *)
  tgxClear( P1 );
  tgxRefresh( P1 );
  tgxShow( P1, 0 );
  tgxClear( P2 );
  tgxRefresh( P2 );
  tgxShow( P2, 0 );
  for CX:=1 to 9 do             (* Use CX as temp variable *)
    P3^.Chars^[CX-1] := METER_DATA[CX];
  tgxRefresh( P3 );             (* Window is refreshed but NOT shown *)

  (* Initialize global variables *)
  CX := P1^.PixelWidth shr 1;   (* Center coords *)
  CY := P1^.PixelHeight shr 1;
  RX := CX-2;                   (* Radius *)
  RY := CY-2;
  MeterCount := 0;
  MeterDelta := 1;

  (* Main loop *)
  GotoXY( P1^.X+1, P1^.Y+P1^.Height+1 ); Write( 'Clock' );
  GotoXY( P2^.X+1, P2^.Y+P2^.Height+1 ); Write( 'Wavey message' );
  GotoXY( P3^.X+1, P3^.Y+P3^.Height+1 ); Write( 'Meter' );
  GotoXY( 1, 24 );
  Write( 'This program runs in text mode...' );
  repeat
    GetTime( Hour, Min, Sec, CS );
    DrawClock( P1, Hour, Min, Sec );    (* Clock *)
    DrawMessage( P2 );                  (* Message *)
    Inc( MeterCount );
    DrawAndShowMeter( P3, MeterCount ); (* Meter *)
    tgxWaitRetrace;
    tgxRefresh( P1 );
    tgxRefresh( P2 );
  until( KeyPressed );
  while( KeyPressed ) do C := ReadKey;

  (* Destroy windows *)
  ClrScr;
  tgxDestroy( P1 );
  tgxDestroy( P2 );
  tgxDestroy( P3 );

  (* Terminate the library and exit to DOS *)
  tgxDone;
end.
